<#import "../../application/docbook+xml/utils.ftl" as utils>
<#assign rootEntity = entity.root>
<#-- Utility list to remember tarversed entities -->
<#assign traveresedEntityKeyList = [rootEntity.documentKey]>
<#-- Utility list to remember entities yet to be traversed -->
<#assign toTravereseEntityList = []>
<#-- Print CSV header row -->
<@printCsvHeadersRow />
<#-- Traverse the main entity -->
<@traverseEntity rootEntity />
<#-- Go throught the found referenced entities and traverse them. NOTE: the list can grow further as entities can reference further entities. Unclear whether this works well in FreeMarker -->
<#-- FreeMarker does not support do-while loops as such do this workaround with maxLoop -->
<#assign maxLoop=5>
<#list 1..maxLoop as x>
  <#-- Remove documents that have been already traversed -->
  <#assign toTravereseEntityList = utils.removeDocumentWitKey(toTravereseEntityList, traveresedEntityKeyList) >
  <#if !(toTravereseEntityList?has_content)>
    <#-- Stop looping if there is nothing to traverse -->
    <#break>
  </#if>
  <#-- Travers each entity from the list -->
  <#list toTravereseEntityList as entityItem>
    <#assign traveresedEntityKeyList= traveresedEntityKeyList + [entityItem.documentKey]>
    <@traverseEntity entityItem />
  </#list>
</#list>


<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->

<#macro traverseEntity entityDoc>
  <#if utils.isCompositeEntityDocument(entityDoc)>
    <@traverseCompositeEntity entityDoc />
  <#else>
    <@traverseSimpleEntity entityDoc />
  </#if>
</#macro>

<#macro traverseSimpleEntity entityDoc>
  <@traverseDoc entityDoc entityDoc 0 />
</#macro>

<#macro traverseCompositeEntity entityDoc>
  <#local tableOfContents=iuclid.localizeTreeFor(entityDoc.documentType, entityDoc.submissionType, entityDoc.documentKey, utils.getInheritedTemplates(entityDoc), utils.getRelatedCategories(entityDoc), utils.getRelatedMixtures(entityDoc))>
  <@traverseToc tableOfContents entityDoc 0 />
</#macro>

<#macro traverseToc sectionNode entityDocument level=0>
  <#if sectionNode?has_content>
    <#-- iterate through the section documents under this section if there are any -->
    <#if sectionNode.content?has_content>
      <#list sectionNode.content as doc>
        <#-- traverse through the document's blocks and fields aka elements -->
        <@traverseDoc doc entityDocument level+1 sectionNode />
      </#list>
    </#if>
    <#-- iterate through the child sections of the node and do a recursive call -->
    <#if sectionNode?children?has_content>
      <#list sectionNode?children as child>
        <@traverseToc child entityDocument level+1 />
      </#list>
    </#if>
  </#if>
</#macro>

<#macro traverseDoc docElementNode entityDocument level=0 sectionNode="" isDocument=true>
  <#if docElementNode?has_content>
    <#-- Annotations are only at document level -->
    <#if isDocument>
      <@printAnnotationsForDocument docElementNode entityDocument sectionNode />
    </#if>
    <#-- Check if the element node is a reference field (single or multiple) and remember referenced entities for later traversal in toTravereseEntityList -->
    <#assign toTravereseEntityList = utils.checkAndRememberReferencedEntity(docElementNode, toTravereseEntityList, traveresedEntityKeyList) />
    <#-- iterate through the child structure elements of this document element if there are any and do a recursive call -->
    <#if docElementNode?children?has_content>
      <#list docElementNode?children as child>
        <@traverseDoc child entityDocument level+1 sectionNode false/>
      </#list>
    </#if>
  </#if>
</#macro>

<#macro printAnnotationsForDocument sectionDoc entityDocument sectionNode>
  <#local sectionName = "">
  <#if sectionNode?has_content>
    <#if sectionNode.number?has_content>
      <#local sectionName = sectionNode.number + " ">
    </#if>
    <#local sectionName = sectionName + sectionNode.title>
  </#if>
  <#local sectionDocType = sectionDoc.documentType>
  <#if !utils.isEntityDocument(sectionDoc)>
    <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
  </#if>
  <#-- The web URL pointing to the document -->
  <#local docUrl=utils.getDocumentUrl(sectionDoc) />
  <#-- Get the annotations of the document and print a row for each of them -->
  <#local annotationKeys = sectionDoc.annotations />
  <#if annotationKeys?has_content>
    <#list annotationKeys as key>
      <#local annotation = iuclid.getDocumentForKey(key) />
      <#local annotationStatus = utils.getPicklistFieldAsText(annotation.AdminInfo.AnnotationStatus) />
      <#local annotationAgreement = utils.getPicklistFieldAsText(annotation.EvalInfo.AgreementWithApplicantsSummary) />
<@utils.csvValue annotation.AdminInfo.Name/>,<@utils.csvValue annotationStatus/>,<@utils.csvValue annotationAgreement/>,<@utils.csvValue annotation.EvalInfo.Remarks/>,<@utils.csvValue entityDocument.name/>,${entityDocument.documentType},${entityDocument.documentKey},<@utils.csvValue sectionName/>,<@utils.csvValue sectionDoc.name/>,${sectionDocType},${sectionDoc.documentKey},<@utils.csvValue docUrl/>
    </#list>
  </#if>
</#macro>

<#macro printCsvHeadersRow>
"Annotation name","Annotation status","Annotation - Agreement with applicant","Annotation - Evaluation remarks","Entity name","Entity type","Entity UUID","TOC Section name","Section document name","Section document type","Section document UUID","URL"
</#macro>
